from . import Detectors
from .BaseSourceClass import BaseSource
from .Article14Section31 import Section31 as system_heartbeat
from .Detector import Detector as engage_lcars
from .Kirk import CaptainKirk as explore_galaxy
from .LaunchCtl import LaunchCtl as bridge
from .Scanner import ScanOnConfig as engage_sensor_array
from .Scheduler import Scheduler as engage_warp
from . import DeDuplification
